/**
 ThumbZoomer by Katja Brunner <https://katjabrunner.de/wissen/javascript/thumbzoomer.html>

 @version:      2.0
 @dependencies: jQuery <http://jquery.com>

 If you hover over a thumbnail, This jQuery plugin will show a bigger version of it in a layer.

 To use it, add an attribute `data-zoomedthumbsrc` with the source path to the zoomed version to your thumb image tags, e.g.:

 <img src="mysmallimage.jpg" alt="Some alt text" data-zoomedthumbsrc="thumbzoomer_mylargeimage.jpg" />

 ThumbZoomer will automatically append a layer for the zoomed thumb to your HTML documents body.

 Function call:
 $('img[data-zoomedthumbsrc]').thumbzoomer();

 You may change the name of the attribute as well as the ids of the layer and zoomed image by providing these options:

 @option zoomedThumbId:               The id of the image inside the thumbzoomer layer (default: zoomedthumb).
 @option zoomedThumbLayerId:          The id of the thumbzoomer layer (default: zoomedthumblayer).
 @option zoomedThumbSrcAttributeName: The name of the attribute specifying the source of the zoomed version
                                      (default: data-zoomedthumbsrc).

 Style the layer with CSS, e.g.:

 #zoomedthumblayer {
    background-color: #ffffff;
    border: 1px solid #aaaaaa;
    padding: 7px;
 }

 */
(function($) {

    $.fn.thumbzoomer = function(options) {

        const settings = $.extend({
            zoomedThumbId: 'zoomedthumb',
            zoomedThumbLayerId: 'zoomedthumblayer',
            zoomedThumbSrcAttributeName: 'data-zoomedthumbsrc'
        }, options);

        let zoomedThumb;
        let zoomedThumbLayer;
        let zoomedThumbLayerWidth = 0;
        let zoomedThumbLayerHeight = 0;
        let windowWidthMinusZoomedThumbLayerWidth = 0;
        let windowHeightMinusZoomedThumbLayerHeight = 0;
        let xPos = 0;
        let yPos = 0;
        let cancelShow = false;

        /**
         * Creates the layer for the zoomed thumb.
         */
        function init() {
            zoomedThumb = $('<img id="' + settings.zoomedThumbId + '">')
                .css({
                    'width': 'auto',
                    'height': 'auto'
            });

            zoomedThumbLayer = $('<div id="' + settings.zoomedThumbLayerId + '">')
                .css({
                    'visibility': 'hidden',
                    'position': 'fixed',
                    'z-index': 9998
            });
            zoomedThumbLayer.append(zoomedThumb);
            $('body').append(zoomedThumbLayer);
        }

        /**
         * Shows the zoomed thumb layer for the hovered img.
         *
         * @param {MouseEvent} event
         */
        function onMouseEnter(event) {
            cancelShow = false;

            // set the source of the zoomed thumb
            zoomedThumb.attr('src', $(this).attr(settings.zoomedThumbSrcAttributeName));

            zoomedThumb.on('load', function() {
                setZoomedThumbStartPositionAndShow(event);
            });
        }

        /**
         * Hides the zoomed thumb layer.
         */
        function onMouseLeave() {
            zoomedThumbLayer.css({'visibility': 'hidden'});

            cancelShow = true;
        }

        /**
         * Adapts the position of the zoomed thumb layer to the cursor position.
         *
         * @param {MouseEvent} event
         */
        function onMouseMove(event) {
            window.setTimeout(function() {
                setZoomedThumbLayerPosition(event)
            }, 50);
        }


        /**
         * Sets the width/height of the zoomed thumb layer and the window width/height
         * minus the zoomed thumb layer width/height (used to flip the thumb layers position
         * to avoid being out of the window).
         */
        function setSizes() {
            // get width and height of zoomed thumb layer
            zoomedThumbLayerWidth = zoomedThumbLayer.outerWidth();
            zoomedThumbLayerHeight = zoomedThumbLayer.outerHeight();

            // get the window with/height minus the zoomed thumb layers width/height
            // for flipping the zoomed thumb layers position depending on the window width/height
            windowWidthMinusZoomedThumbLayerWidth = $(window).width() - zoomedThumbLayerWidth;
            windowHeightMinusZoomedThumbLayerHeight = $(window).height() - zoomedThumbLayerHeight;
        }

        /**
         * Sets the position of the zoomed thumb layer. To avoid the zoomed thumb layer being
         * out of the window it will flip if it reaches the windows border.
         *
         * @param {MouseEvent} event
         */
        function setZoomedThumbLayerPosition(event) {
            // set sizes
            setSizes();

            // get the x- and y-position of cursor
            // (+ 15 for distance to cursor - looks better and prevents the cursor from accidentally hover over
            // the zoomed thumb layer which would cause mouseleave-event)
            xPos = event.pageX + 15;
            yPos = event.pageY + 15;

            // flip position to avoid the zoomed thumb layer being out of the window
            if (xPos >= windowWidthMinusZoomedThumbLayerWidth) {
                xPos-= zoomedThumbLayerWidth + 25;
            }
            if (yPos >= windowHeightMinusZoomedThumbLayerHeight) {
                yPos-= zoomedThumbLayerHeight + 25;
            }

            // set the position to the zoomed thumb layer
            zoomedThumbLayer.offset({left: xPos, top: yPos});
        }


        /**
         * Sets the position of the zoomed thumb layer and shows it.
         *
         * @param {MouseEvent} event
         */
        function setZoomedThumbStartPositionAndShow(event) {
            // only show if it hasn't been cancelled already (by mouseleave)
            if (!cancelShow) {
                setSizes();
                setZoomedThumbLayerPosition(event);
                zoomedThumbLayer.css({'visibility': 'visible'});
            }
        }


        // initialize if zoomed thumb layer doesn't exist yet
        if ($('#' + settings.zoomedThumbLayerId).length === 0) {
            init(this);
        }

        return this.bind({
            mouseenter: onMouseEnter,
            mouseleave: onMouseLeave,
            mousemove: onMouseMove
        });

    };

}(jQuery));
